<?php
namespace Feedier;

use Feedier\Admin\Feedbacks;

/**
 * Class Admin
 *
 * @package Feedier
 */
class Admin extends Main
{
	/**
	 * Feedier constructor.
	 *
	 * The main plugin actions registered for WordPress
	 */
	public function __construct()
	{
		add_action('admin_menu',                    array($this, 'addAdminMenu'));
		add_action('wp_ajax_store_admin_data',      array($this, 'storeAdminData'));
		add_action('admin_enqueue_scripts',         array($this, 'addAdminScripts'));
	}

	/**
	 * Callback for the Ajax request
	 *
	 * Updates the options data
	 *
	 * @return void
	 */
	public function storeAdminData()
	{

		if (wp_verify_nonce($_POST['security'], $this->_nonce ) === false) {
			die('Invalid Request! Reload your page please.');
        }

		$data = $this->getData();

		foreach ($_POST as $field=>$value) {

			if (substr($field, 0, 8) !== 'feedier_') {
				continue;
			}

			if (empty($value)) {
				unset($data[$field]);
            }

			// We remove the feedier_ prefix to clean things up
			$field = substr($field, 8);

			// Sanitize
            if (is_array($value)) {
                foreach ($value as $key=>$sub_value) {
	                if (is_array($sub_value)) {
                        foreach ($sub_value as $sub_key=>$sub_sub_value) {
	                        $sub_value[$sub_key] = sanitize_text_field($sub_sub_value);
                        }
	                } else {
		                $value[$key] = sanitize_text_field($sub_value);
                    }
                }
            } else {
	            $value = sanitize_text_field($value);
            }

			$data[$field] = $value;

		}

		update_option($this->option_name, $data);

		echo __('Saved!', 'feedier');
		die();

	}

	/**
	 * Adds Admin Scripts for the Ajax call
	 */
	public function addAdminScripts()
	{
		wp_enqueue_style('feedier-admin', FEEDIER_URL. 'assets/css/admin.css', false, 1.0);
		wp_enqueue_script('feedier-admin', FEEDIER_URL. 'assets/js/admin.js', array(), 1.0);

		$admin_options = array(
			'ajax_url' => admin_url( 'admin-ajax.php' ),
			'_nonce'   => wp_create_nonce( $this->_nonce ),
		);

		wp_localize_script('feedier-admin', 'feedier_exchanger', $admin_options);
	}

	/**
	 * Adds the Feedier label to the WordPress Admin Sidebar Menu
     *
     * It also initiates the Feedback class to fire the WP list table at the right time
	 */
	public function addAdminMenu()
	{
		$page_hook = add_options_page(
            'Feedier feedbacks',
            'Feedier feedbacks',
			'manage_options',
			'feedier',
			array($this, 'adminLayout')
        );

		add_action('load-'. $page_hook, function () {
			$arguments = array(
				'label'		=>	__( 'Feedbacks Per Page', 'feedier'),
				'default'	=>	30,
				'option'	=>	'feedbacks_per_page'
			);
			add_screen_option( 'per_page', $arguments);
			new Feedbacks('feedier');
        });
	}

	/**
	 * Make an API call to the Feedier API and returns the response
	 *
	 * @param $private_key string
	 * @param $carrier_id int|null
	 *
	 *
	 * @return array
	 */
	private function getCarriers($private_key, $carrier_id = null)
	{
		$data       = array();
		$carrier_id = ($carrier_id) ? '&carrier_id='. $carrier_id : '';
		$response   = wp_remote_get(FEEDIER_PROTOCOL. '://api.'. FEEDIER_ENDPOINT .'/v1/carriers/?api_key='. $private_key . $carrier_id);

		if (is_array($response) && !is_wp_error($response)) {
			$data = json_decode($response['body'], true);
		}

		return $data;
	}

	/**
	 * Get a Dashicon for a given status
	 *
	 * @param $valid boolean
	 *
	 * @return string
	 */
	private function getStatusIcon($valid)
	{
		return ($valid) ? '<span class="dashicons dashicons-yes success-message"></span>' : '<span class="dashicons dashicons-no-alt error-message"></span>';
	}

	/**
	 * Outputs the Admin Dashboard layout containing the form with all its options
	 *
	 * @return void
	 */
	public function adminLayout()
	{

		$data                   = $this->getData();

		$data['private_key']    = (isset($data['private_key'])) ? $data['private_key'] : null;
		$api_response           = $this->getCarriers($data['private_key']);
		$not_ready              = (empty($data['public_key']) || empty($api_response) || isset($api_response['error']));
		$has_engager_preview    = (isset($_GET['feedier-demo-engager']) && $_GET['feedier-demo-engager'] === 'go');
		$has_wc                 = (class_exists('WooCommerce'));
		$nav_tab                = (isset($_GET['tab'])) ? sanitize_text_field($_GET['tab']) : false;
		$tabs                   = array(
			'feedier-settings'      => __('Feedier connection', 'feedier'),
			'feedier-rules'         => __('Widget display rules', 'feedier'),
			'feedier-design'        => __('Widget design', 'feedier'),
			'feedier-woocommerce'   => __('WooCommerce email', 'feedier'),
			'feedier-feedback'      => __('Feedbacks', 'feedier'),
			'feedier-insights'      => __('Dashboard (manage forms and reports) ↗️', 'feedier'),
		);

		if ($nav_tab === 'feedier-design') {
			$tab_class = new Admin\WidgetDesign();
		} else if ($nav_tab === 'feedier-rules') {
			$tab_class = new Admin\WidgetRules();
		} else if ($nav_tab === 'feedier-woocommerce') {
			$tab_class = new Admin\WooCommerce();
		} else if ($nav_tab === 'feedier-feedback') {
			$tab_class = new Admin\Feedbacks();
		} else {
			$tab_class = new Admin\Settings();
			$nav_tab = 'feedier-settings';
		}

		?>

		<div class="wrap">

			<h1><?php _e('Feedier - collect real-time feedback', 'feedier'); ?></h1>

			<h2 class="nav-tab-wrapper">
				<?php foreach ($tabs as $key=>$tab) { ?>
					<?php if (!$has_wc && $key === 'feedier-woocommerce') {
						continue;
					} ?>

                    <?php if ($key === 'feedier-insights') : ?>
                        <a href="https://dashboard.<?php echo FEEDIER_ENDPOINT; ?>/#/" target="_blank" class="nav-tab">
							<?php echo $tab; ?>
                        </a>
                    <?php else: ?>
                        <a href="?page=feedier&tab=<?php echo $key; ?>" class="nav-tab <?php echo ($nav_tab && $nav_tab === $key) ? 'nav-tab-active' : ''; ?>">
							<?php echo $tab; ?>
                        </a>
                    <?php endif; ?>
				<?php } ?>
			</h2>

			<?php if ($has_engager_preview) { ?>
				<?php $widget_Engager = new Engager();
				$widget_Engager->addFooterCode(true); ?>
				<p class="notice notice-warning p-10">
					<?php _e( 'The demo engager is enabled. You will see the widget, exactly as it will be displayed on your site.<br> The only difference is that until the preview is turned off it will always come back compared to the live version.', 'feedier' ); ?>
				</p>
			<?php } ?>


			<form id="feedier-admin-form" class="postbox">

				<div class="form-group inside">

					<h2>
						<?php echo $this->getStatusIcon($tab_class->isActive($not_ready, $data)); ?>
						<?php echo $tab_class->getTitle(); ?>
					</h2>

                    <?php
                    // If we have an error returned by the API
                    if (isset($api_response['type']) && $api_response['type'] === 'error'): ?>

                        <p class="notice notice-error">
                            <?php echo $api_response['message']; ?>
                        </p>

                    <?php endif; ?>

		            <?php $tab_class->writeMarkup($not_ready, $data, $api_response); ?>

				</div>

				<hr>

				<div class="inside">

					<button class="button button-primary" id="feedier-admin-save" type="submit">
						<?php _e( 'Save', 'feedier' ); ?>
					</button>

					<?php if (!$not_ready && $nav_tab === 'feedier-engager'): ?>

						<?php if ($has_engager_preview): ?>
							<a href="<?php echo admin_url('admin.php?page=feedier'); ?>" class="button">
								<?php _e( 'Stop engager widget', 'feedier' ); ?>
							</a>
						<?php else: ?>
							<a href="<?php echo add_query_arg('feedier-demo-engager','go'); ?>" class="button">
								<?php _e( 'Preview engager widget', 'feedier' ); ?>
							</a>
						<?php endif; ?>

					<?php endif; ?>

				</div>

			</form>

		</div>

		<?php

	}

}